//
//  CropMarker.h
//  Cropped Image
//
//  Created by jcr on Tue Jul 16 2002.
//  Copyright (c) 2002 Apple Computer, Inc.  All rights reserved.
//

#import <Cocoa/Cocoa.h>
#import <AddressBook/AddressBook.h>
#import "MMSPersistentObject.h"
#import "MMSPerson.h"
#import "MMSPersonLabelViewOwner.h"


#define SKT_HALF_HANDLE_WIDTH 4.0
#define SKT_HANDLE_WIDTH (SKT_HALF_HANDLE_WIDTH * 2.0)

typedef enum {
    NoKnob = 0,
    UpperLeftKnob,
    UpperMiddleKnob,
    UpperRightKnob,
    MiddleLeftKnob,
    MiddleRightKnob,
    LowerLeftKnob,
    LowerMiddleKnob,
    LowerRightKnob,
} knobLocation;

enum {
    NoKnobsMask = 0,
    UpperLeftKnobMask = 1 << UpperLeftKnob,
    UpperMiddleKnobMask = 1 << UpperMiddleKnob,
    UpperRightKnobMask = 1 << UpperRightKnob,
    MiddleLeftKnobMask = 1 << MiddleLeftKnob,
    MiddleRightKnobMask = 1 << MiddleRightKnob,
    LowerLeftKnobMask = 1 << LowerLeftKnob,
    LowerMiddleKnobMask = 1 << LowerMiddleKnob,
    LowerRightKnobMask = 1 << LowerRightKnob,
    AllKnobsMask = 0xffffffff,
};

/*typedef enum _CropMarkerStyle
  {
  plainMarkerStyle,
  finderMarkerStyle,
  iPhotoMarkerStyle,
  lassoMarkerStyle
  } CropMarkerStyle; 
*/

typedef enum _SelectionTrackingMode
	{
	trackNone,
	trackSelecting,
	trackMoving,
	trackResizing  // Not implemented!  (Currently left as an exercise for the reader, but I may fill this in someday.)
	} SelectionTrackingMode;
	
@interface CropMarker : MMSPersistentObject
{
	NSView *target;
    BOOL selecting, dragging, resizing;
    NSRect selectedRect;
    NSPoint lastLocation;
    NSColor *fillColor, *strokeColor;
	SelectionTrackingMode trackingMode;
	NSBezierPath *selectedPath;
    BOOL isVisible;
    NSInteger activeKnob;
    ABRecord *personRecord;
    NSInteger tagID;
    NSData * jpegCroppedImageData;
    CGFloat scaleFactor;
    NSRect storedFullSizeCroppingRect;
    MMSPersonLabelViewOwner *personLabelSubviewOwner;
    MMSPerson *person;
    
    BOOL hasSpecificLocation;
}


	// Convenience constructor.  Use this in most cases.
+ cropMarkerForView:aView;

+ (id)cropMarkerFromDictionary:(NSDictionary *)aDict
                       forView:(NSView *)aView;

+ (NSCursor *)diagonalResizeCursor1;
+ (NSCursor *)diagonalResizeCursor2;


	// Designated Intiailizer
- initWithView:(NSView *) aView;
- (void)configureForView:(NSView *)aView;

- (void) drawCropMarker;
- (void)configureCursor;

	// The mouse-tracking methods
+ (NSInteger)flipKnob:(NSInteger)knob horizontal:(BOOL)horizFlag;
- (void) startSelectingAtPoint:(NSPoint) where;
- (void) continueSelectingAtPoint: (NSPoint) where;
- (void) stopSelectingAtPoint:(NSPoint) where;
- (void) startMovingAtPoint:(NSPoint) where;
- (void) continueMovingAtPoint: (NSPoint) where;
- (void) stopMovingAtPoint:(NSPoint) where;
- (void)startResizingAtPoint:(NSPoint)where;
- (void)stopResizingAtPoint:(NSPoint) where;

- (void) mouseDown:(NSEvent *) theEvent;
- (void)mouseEntered:(NSEvent *)theEvent;
- (void) mouseUp:(NSEvent *) theEvent;
- (void) mouseDragged:(NSEvent *) theEvent;

 // Simple Accessors
- (void) setColor:(NSColor *) color;
- (void) setFillColor:(NSColor *) color;
- (void) setStrokeColor:(NSColor *) color;
- (NSBezierPath *) selectedPath;
- (NSRect) selectedRect;
- (void) setSelectedRect:(NSRect) rect;
- (void) setSelectedRectOrigin:(NSPoint) where;
- (void) setSelectedRectSize:(NSSize) size;
- (void) moveSelectedRectBy:(NSSize) delta;
- (SelectionTrackingMode)trackingMode;
- (NSColor *)strokeColor;
- (void)setIsSelected:(BOOL)aChoice;
- (BOOL)isSelected;
- (BOOL)isVisible;
- (NSData *)jpegCroppedImageData;
- (void)setJpegCroppedImageData:(NSData *)aJpegCroppedImageData;
- (NSData *)_jpegCroppedImageData;
- (void)setIsVisible:(BOOL)flag;
- (NSInteger)tagID;
- (void)setTagID:(NSInteger)aTagID;
- (NSView *)target;
- (void)setTarget:(NSView *)aView;
- (MMSPerson *)person;
- (void)setPerson:(MMSPerson *)aPerson;
- (ABRecord *)personRecord;
- (void)setPersonRecord:(ABRecord *)aPersonRecord;
- (CGFloat)scaleFactor;
- (void)setScaleFactor:(CGFloat)aScaleFactor;
- (BOOL)hasSpecificLocation;
- (void)setHasSpecificLocation:(BOOL)value;


// Selection Handles support
- (NSUInteger)knobMask;
- (NSInteger)knobUnderPoint:(NSPoint)point;
- (void)drawHandleAtPoint:(NSPoint)point inView:(NSView *)view;
- (void)drawHandlesInView:(NSView *)view;
- (NSInteger)activeKnob;
- (void)setActiveKnob:(NSInteger)aKnob;

//
// Drawing
//
- (void)updatePersonLabelViewPosition;

// Resizing Support
- (void)adjustForTargetView;
- (NSInteger)resizeByMovingKnob:(NSInteger)knob 
                  toPoint:(NSPoint)point;
- (void)targetViewBoundsChanged:(NSNotification *)notification;
- (void)targetViewFrameChanged:(NSNotification *)notification;

// Serialization methods
- (NSDictionary *)dataDict;
- (NSImage *)croppedImage;
- (NSData *)jpegCroppedImageData;

// Address Book support;
- (void)postPersonLabelNeededNotification;
- (NSString *)personName;
- (void)setPersonLabelSubviewOwner:(MMSPersonLabelViewOwner *)aPersonLabelSubviewOwner;
- (MMSPersonLabelViewOwner *)personLabelSubviewOwner;

// Full Size Image Cropping 
- (NSRect)derivedFullSizeCroppingRect;
- (NSRect)selectionRectForCurrentDisplay;
- (NSRect)storedFullSizeCroppingRect;
- (void)setStoredFullSizeCroppingRect:(NSRect)aStoredFullSizeCroppingRect;


@end

NSRect rectFromPoints(NSPoint p1, NSPoint p2);  // Given two corners, make an NSRect.
