//
//  DataCrux.h
//  DataCrux
//
//  Created by Scott Stevenson on Wed Sep 03 2003.
//  Copyright (c) 2003 Tree House Ideas. All rights reserved.
//

#import <Cocoa/Cocoa.h>
#import "DataCrux-Types.h"

@class DC_SearchDefinition, THI_PipelineController;
@class DC_SQLDatabase;

extern NSString *DCGeneralSQLErrorNotification;


@interface DataCrux : NSObject {

    THI_PipelineController  * _pipelineController;
    NSString                * clientBundleID;
    NSDictionary            * _clientProperties;
    BOOL                    _transactionIsActive;
    NSMutableDictionary     *bindableArrayStorage;    
    
    
    // the use of DC_SQLDatabase is deprecated, but
    // the number of pointers will probably stay the same
    // for a while
    
    DC_SQLDatabase          * keyDatabase;
    DC_SQLDatabase          * diskDatabase;
    DC_SQLDatabase          * memoryDatabase;
}


- (NSDictionary *) bindableArrays;


// startup and shutdown

- (id)          initWithClientID:               (NSString *)clientID;
+ (id)          crux;
+ (id)          cruxWithClientID:               (NSString *)clientID;
- (NSString *)  clientID;
- (void)        setClientID:                    (NSString *)newClientID;

- (id)          initWithClientProperties:       (NSDictionary *)theProperties
                         andDatabasePath:       (NSString *)thePath;


+ (id)          inMemoryCruxWithContentsOfPath: (NSString *)fileName
                           andClientProperties: (NSDictionary *)dataCruxConfig;


// transactions - experimental

- (void) createTransaction;
- (BOOL) executeTransaction;
- (void) cancelTransaction;
- (BOOL) transactionIsActive;



// public interface

- (BOOL)        storeItem:      (id)item;
- (BOOL)        storeItems:     (NSArray *)items;
- (BOOL)        updateItem:     (id)item;
- (BOOL)        deleteItem:     (id)item;
- (BOOL)        deleteItems:    (NSArray *)items;
- (NSArray *)   itemsOfType:    (NSString *)theType;
- (NSArray *)   itemsOfType:    (NSString *)theType
            withConstraints:    (NSDictionary *)mask;
- (NSArray *)   itemsOfType:    (NSString *)theType
       withSearchDefinition:    (DC_SearchDefinition *)searchDefinition;
- (NSInteger) itemCountForType: (NSString *)theType
    withSearchDefinition: (DC_SearchDefinition *)searchDefinition;

- (NSArray *) rawRowsOfType: (NSString *)theType 
       withSearchDefinition: (DC_SearchDefinition *)searchDefinition
              attributeKeys:(NSArray *)attributeKeys;

- (NSArray *)rawRowsOfType:(NSString *)theType 
		withSQLWhereClause:(NSString *)aWhereClause
			 attributeKeys:(NSArray *)attributeKeys;

- (NSArray *)rawSQLOutputFromSQLQueryString:(NSString *)aSQLString;

- (NSArray *)itemsOfType:(NSString *)theType 
      withSQLWhereClause:(NSString *)aSQLWhereClause;

- (NSInteger)itemCountForType: (NSString *)theType 
    withSearchDefinition: (DC_SearchDefinition *)searchDefinition;

- (id)          newItemOfType:  (NSString *)theType;


// to add a new field

- (BOOL) registerAttribute: (NSString *)attribute withType: (NSString *)theType;


// in-memory databases

- (BOOL) copyInMemoryDatabaseToPath: (NSString *)thePath;
- (BOOL) moveInMemoryDatabaseToPath: (NSString *)thePath;
- (BOOL) databaseIsInMemory;


// bindings

- (BOOL) observeChangesToItem: (id)item;






// deprecate?
- (NSString *)dbfilePath;


@end
