//
//  MMSLicenseCodec.h
//  KRMTester
//
//  Created by John Fox on 1/6/06.
//  Copyright 2006 GroupSmarts, LLC. All rights reserved.
//

#import <Cocoa/Cocoa.h>
#import "KagiGenericACG.h"

typedef enum {
    MMSCommercialRegistrationType = 0,
    MMSEducationalRegistrationType,
    MMSEvaluationRegistrationType,
    MMSRetailRegistrationType,
    MMSRetailWinRegistrationType,
    MMSCommercialWinRegistrationType,

} MMSRegistrationType;

@interface MMSLicenseCodec : NSObject 
{
    KagiGenericACG *commercialACG;
    KagiGenericACG *educationalACG;
    KagiGenericACG *evaluationACG;
    KagiGenericACG *retailACG;
    KagiGenericACG *retailWinACG;
    KagiGenericACG *commercialWinACG;
}

//
// Singleton Instance
//
+ (MMSLicenseCodec *)sharedCodec;

//
// Config
//
+ (NSString *)commercialACGConfigString;
+ (NSString *)educationalACGConfigString;
+ (NSString *)evaluationACGConfigString;
+ (NSString *)retailACGConfigString;
+ (NSString *)retailWinACGConfigString;
+ (NSString *)commercialWinACGConfigString;

//
// Decoding
//

- (KagiGenericACG *)acgForRegString:(NSString *)aRegCode;
- (NSDictionary *)infoDictFromRegCode:(NSString *)aRegCode;
- (NSCalendarDate *)dateFromRegCode:(NSString *)aRegCode;

- (BOOL)isValidRegistrationCode:(NSString *)aRegCode
                       forEmail:(NSString *)anEmail
                        andName:(NSString *)aName;

- (BOOL)isValidRetailRegistrationCode:(NSString *)aRegCode;

- (BOOL)isEvaluationRegCode:(NSString *)aRegCode;
- (BOOL)isExpiredRegCode:(NSString *)aRegCode;
- (BOOL)isRetailRegCode:(NSString *)aRegCode;
- (BOOL)isWindowsRegCode:(NSString *)aRegCode;
- (NSString *)prefixFromRegCode:(NSString *)aRegCode;
- (NSString *)constantStringFromRegCode:(NSString *)aRegCode;
- (NSCalendarDate *)expirationForRegCode:(NSString *)aRegCode;


//
// Accessors
//

- (KagiGenericACG *)commercialACG;
- (void)setCommercialACG:(KagiGenericACG *)aKagiACG;

- (KagiGenericACG *)commercialWinACG;
- (void)setCommercialWinACG:(KagiGenericACG *)anACG;

- (KagiGenericACG *)educationalACG;
- (void)setEducationalACG:(KagiGenericACG *)anEducationalACG;

- (KagiGenericACG *)evaluationACG;
- (void)setEvaluationACG:(KagiGenericACG *)anEvaluationACG;

- (KagiGenericACG *)retailACG;
- (void)setRetailACG:(KagiGenericACG *)aRetailACG;

- (KagiGenericACG *)retailWinACG;
- (void)setRetailWinACG:(KagiGenericACG *)anACG;

@end
