//
//  MMSPerson.h
//  MemoryMiner
//
//  Created by John Fox on 10/5/04.
//  Copyright 2004 MemoryMiner Software. All rights reserved.
//

#import <Foundation/Foundation.h>
#import <AddressBook/AddressBook.h>

#import "MMSPersistentObject.h"
//#import "MMSPersonLink.h"

@class MMSPhoto;

typedef enum
{
    unknown = -2,
    current = -1,
    newborn,
    infant,
    toddler,
    child,
    preteen,
    teen,
    youngAdult,
    adult,
    middleAge,
    senior,
} MMSPersonLifePeriodCode;

@interface MMSPerson : MMSPersistentObject {
    NSDate *birthDate;
    NSString *birthDateString;
    NSDate *deceasedDate;
    NSString *deceasedDateString;
    NSString *firstName;
    NSString *lastName;
    NSString *middleName;
    NSString *abUID;
    NSString *gender;
    
    ABPerson *personABRecord;
    NSImage *personImage;
    NSImage *newbornIconImage;
    NSImage *infantIconImage;
    NSImage *toddlerIconImage;
    NSImage *childIconImage;
    NSImage *preTeenIconImage;
    NSImage *teenIconImage;
    NSImage *youngAdultIconImage;
    NSImage *adultIconImage;
    NSImage *middleAgeIconImage;
    NSImage *seniorIconImage;
}

//
// Configuration
//
- (void)configureABPersonRecord;

// Conveniences
+ (NSArray *)abPeopleMatchingSearchWords:(NSArray *)aWordArray
				   filteredByPersonArray:(NSArray *)aPersonArray;

+ (NSArray *)lifePeriodComparisonArray;

+ (MMSPerson *)newOrExistingPersonWithUID:(NSString *)aUID;

+ (MMSPerson *)newPersonFromABRecord:(ABRecord *)anABRecord;

+ (NSArray *)allPersonsSortedByLastName;
+ (NSArray *)allPersonsSortedByFirstName;
+ (NSArray *)allPersonsSortedByFirstOrLastName;
+ (NSArray *)allPersons;
+ (NSArray *)allPersonsSortedByKey:(NSString *)aKey;
+ (MMSPerson *)mePerson;
+ (NSArray *)masterLifePeriodKeyArray;

+ (void)createNewMePerson;
+ (void)invalidateNewMePerson;
+ (void)repairMissingMePerson;
+ (MMSPerson *)personFromLoggedInUser;
+ (BOOL)hasValidatedMePerson;
+ (void)importMePersonRelatives;
+ (NSString *)lifePeriodKeyFromCode:(MMSPersonLifePeriodCode)value;
+ (NSImage *)defaultLifePeriodIcon;
+ (void)updateWSBuddyListWithUserInfoDictArray:(NSArray *)aUserInfoDictArray;
+ (NSArray *)peopleConcurrentWithThesePeople:(NSArray *)aPersonArray;

- (NSArray *)allDestinationRelationhips;
- (NSArray *)allSourceRelationhips;

- (NSArray *)allRelatedPeopleWithClassifiction:(NSString *)aClassification
                                isSourcePerson:(BOOL)isSourcePerson
                                       sortKey:(NSString *)aSortKey;

- (NSArray *)concurrentPeople;

- (id)linkToMePerson;

- (NSString *)localizedGenderName;

- (NSArray *)locationTimeMarkers;

- (NSInteger)genderIndex;

- (NSArray *)ownedPhotos;

- (NSArray *)selectionMarkers;
- (NSArray *)selectionMarkersWithThumbnails;

- (NSArray *)photosSortedByDate;
- (NSArray *)activePhotosSortedByDate;
- (NSArray *)photosSortedByDateShowingActiveOnly:(BOOL)showActiveOnly;

- (NSArray *)uniqueLocations;
- (NSArray *)uniquePlottableLocations;
- (NSArray *)chronologicalPlottableLocations;

- (NSString *)name;
- (NSString *)nameAndBirthYear;
- (NSString *)nameAndCountString;
- (NSString *)birthYear;

- (NSImage *)iconImage;
- (NSImage *)globalIconImage;
- (BOOL)storeGlobalIconImage:(NSImage *)anImage;
- (void)clearGlobalIconImage;

- (NSArray *)allCustomLifePeriodIconPaths;

- (NSImage *)iconImageForLifePeriodNamed:(NSString *)aName;
- (NSString *)storagePathForLifePeriodIconNamed:(NSString *)aName;
- (NSString *)storagePathForLifePeriodIconNamed:(NSString *)aName
                                  withExtension:(NSString *)anExtension;

- (NSString *)tiffStoragePathForLifePeriodIconNamed:(NSString *)aName;
- (NSString *)jpegStoragePathForLifePeriodIconNamed:(NSString *)aName;

- (BOOL)hasCustomIconForLifePeriodNamed:(NSString *)aName;

- (NSString *)verifiedIconImagePathForLifePeriodNamed:(NSString *)aName;

//
// Web Services Support
//
- (void)updateWSBuddyList;

//
// Timeline
//
- (NSArray *)locationTimeMarkers;

//
// Accessors
//

- (NSImage *)newbornIconImage;
- (void)setNewbornIconImage:(NSImage *)anImage;

- (NSImage *)infantIconImage;
- (void)setInfantIconImage:(NSImage *)anIconImage;

- (NSImage *)toddlerIconImage;
- (void)setToddlerIconImage:(NSImage *)anImage;

- (NSImage *)childIconImage;
- (void)setChildIconImage:(NSImage *)anImage;

- (NSImage *)preTeenIconImage;
- (void)setPreTeenIconImage:(NSImage *)anIconImage;

- (NSImage *)teenIconImage;
- (void)setTeenIconImage:(NSImage *)anImage;

- (NSImage *)youngAdultIconImage;
- (void)setYoungAdultIconImage:(NSImage *)anImage;

- (NSImage *)adultIconImage;
- (void)setAdultIconImage:(NSImage *)anImage;

- (NSImage *)middleAgeIconImage;
- (void)setMiddleAgeIconImage:(NSImage *)anImage;

- (NSImage *)seniorIconImage;
- (void)setSeniorIconImage:(NSImage *)anIconImage;

- (NSInteger)ageInMonthsRelativeToPhoto:(MMSPhoto *)aPhoto;
- (NSArray *)photosFromLifePeriod:(MMSPersonLifePeriodCode)aLifePeriod;
- (MMSPersonLifePeriodCode)lifePeriodCodeForPhoto:(MMSPhoto *)aPhoto;

// Accessors
- (NSString *)abUID;
- (void)setABUID:(NSString *)anABUID;

- (NSDate *)birthDate;
- (void)setBirthDate:(NSDate *)aBirthDate;

- (NSString *)birthDateString;
- (void)setBirthDateString:(NSString *)aBirthDateString;

- (NSDate *)deceasedDate;
- (void)setDeceasedDate:(NSDate *)aDeceasedDate;

- (NSString *)deceasedDateString;
- (void)setDeceasedDateString:(NSString *)aDeceasedDateString;

- (NSString *)firstName;
- (void)setFirstName:(NSString *)aFirstName;

- (NSString *)gender;
- (void)setGender:(NSString *)value;

- (NSString *)lastName;
- (void)setLastName:(NSString *)aLastName;

- (NSString *)middleName;
- (void)setMiddleName:(NSString *)aMiddleName;

- (ABPerson *)personABRecord;
- (void)setPersonABRecord:(ABPerson *)aPersonRecord;
- (ABPerson *)_personABRecord;

- (NSImage *)personImage;
- (void)setPersonImage:(NSImage *)aPersonImage;
- (NSImage *)_personImage;

@end

#define newPersonCreatedNotification @"MMSPersonCreatedNotification"


