//
//  MMSPhoto.h
//  MemoryMiner
//
//  Created by John Fox on 10/4/04.
//  Copyright 2004 MemoryMiner Software. All rights reserved.
//

#import <Foundation/Foundation.h>
#import "MMSPersistentObject.h"
#import "MMSChecksum.h"
#import "MMSPerson.h"
//#import "MMSSelectionMarker.h"
#import "MMSLocation.h"
#import "MMSChecksum.h"
#import "iPhotoLibraryController.h"

@class MMSSelectionMarker;

@interface MMSPhoto : MMSPersistentObject 
{
    NSString *fileURL;
    NSDate *catalogDate;
    NSCalendarDate *creationDate;
    NSString *creationDateString;
    NSNumber *fileSize;
    NSString *yearString;
    NSString *monthString;
    NSString *dayString;
    NSDate *lastUpdate;
    NSString *lowRezURL;
    NSString *notes;
    NSString *thumbnailURL;
    NSString *title;
    NSString *photoLibID;
    MMSPerson *photoOwner;
    MMSLocation *location;
    NSString *photoOwnerID;
    NSString *locationID;
    NSNumber *pixelsWide;
    NSNumber *pixelsHigh;
	NSNumber *gpsAltitude;
	NSNumber *gpsLatitude;
	NSNumber *gpsLongitude;
    NSString *checksumString;
    NSString *originalChecksumString;
    MMSChecksum *checksum;
    NSString *isHidden;
    NSArray *selectionMarkers;
    NSNumber *numberOfPeople;
    NSImage *miniIconImage;
    NSImage *iconImage;
    NSImage *previewImage;
    NSImageRep *iconImageRep;
    NSAttributedString *notesAttributedString;
    NSNumber *exportScaleFactor;
    NSURL *photoThumbnailURL;
    NSString *relativeStoragePath;
    NSDictionary *rawXMPDataDict;
	
    BOOL bitmapCacheIsLoaded;
    BOOL hasMissingSourceFile;
	BOOL hasUnsyncedXMPData;
	
	// Face Detection support
	NSArray *faceInfoDictArray;
	
	NSDictionary *placeInfoDict;
}

@property(retain) NSDictionary *placeInfoDict;

+ (MMSPhoto *)newOrExistingPhotoWithURL:(NSURL *)anURL
                              forOwner:(MMSPerson *)aPhotoOwner
                              infoDict:(NSDictionary *)anInfoDict;

+ (MMSPhoto *)newPhotoFromURL:(NSURL *)anURL
                    forOwner:(MMSPerson *)aPhotoOwner
                    infoDict:(NSDictionary *)anInfoDict;

+ (MMSPhoto *)photoWithURL:(NSURL *)anURL;

+ (NSArray *)allPhotosWithPerson:(MMSPerson *)aPerson
                  sortDescending:(BOOL)aSortChoice;

+ (NSArray *)allPhotosWithNoDate;

+ (NSArray *)allActivePhotosWithNoDate;
+ (NSArray *)allPhotosWithNoDateShowingActiveOnly:(BOOL)showActiveOnly;

+ (NSArray *)allPhotosWithNoLocation;

+ (NSArray *)allPhotosWithNoPeople;
+ (NSArray *)allActivePhotosWithNoPeople;

+ (NSArray *)allPhotosWithNoPeopleShowingActiveOnly:(BOOL)showActiveOnly;

//
// DB Blob based photo bitmap archive support
//
+ (BOOL)hasArchivedBitmapForPhotoID:(NSString *)aPhotoID;
+ (NSData *)archivedBitmapCacheForPhotoID:(NSString *)aPhotoID;
+ (void)storeCachedBitmapData:(NSData *)aBitmapCache 
				 usingPhotoID:(NSString *)aPhotoID;

+ (void)buildBitmapCacheDict;

+ (NSArray *)activePhotosFromArray:(NSArray *)anArray;
+ (NSArray *)allActivePhotos;
+ (NSArray *)allInactivePhotos;
+ (NSArray *)baseAttributeKeys;
+ (NSSet *)inactivePhotoSet;

+ (NSString *)archivedIconImageRepPathForPhotoWithID:(NSString *)anObjectID;
+ (NSImageRep *)archivedIconImageRepForPhotoWithID:(NSString *)anObjectID;
+ (NSString *)JPEGIconPathForObjectID:(NSString *)anObjectID;
+ (void)deleteArchivedIconImageRepForPhotoWithID:(NSString *)anObjectID;
+ (NSImage *)iconImageFromThumbnailURL:(NSURL *)aThumbnailURL;

+ (NSString *)bitmapCachePath;
+ (NSString *)mediumSizeBitmapCachePath;
+ (NSString *)thumbnailCacheDBPath;
+ (NSMutableDictionary *)photoBitmapCacheDict;
+ (NSMutableDictionary *)mediumSizePhotoBitmapCacheDict;
+ (NSMutableDictionary *)photoThumbnailCacheDict;
+ (void)configureMediumBitmapCacheDict;
+ (void)configureBitmapCacheDict;
+ (NSMutableDictionary *)iPhotoKeyedCacheDict;
+ (void)markMediumCacheDictAsDirty;
+ (void)markCacheDictAsDirty;
+ (void)markCacheDictAsClean;
+ (BOOL)mediumCacheDictIsDirty;
+ (BOOL)cacheDictIsDirty;
+ (NSBitmapImageRep *)placeholderImageRep;

+ (CGFloat)reductionToFitSize:(NSSize)aSize
                     inRect:(NSRect)aRect;

+ (NSRect)rectScaledToFitSize:(NSSize)aSize
                       inRect:(NSRect)aRect;

+ (void)storeCachedThumbnailFromURL:(NSURL *)anURL
                        forObjectID:(NSString *)anObjectID;


+ (NSInteger)personCountForPhotoWithID:(NSString *)anID;
+ (NSMutableDictionary *)assignedPersonCountCacheDict;

+ (NSImage *)missingPhotoIcon;
+ (NSImageRep *)missingPhotoIconRep;

+ (NSNumber *)trashCount;

//
// XMP Support
//
+ (void)setXMPPLuginController:(id)aPluginController;
+ (id)xmpPluginController;

//
// Keyword Mapping
//

- (void)performMappingWithKeywords:(NSArray *)aKeywordArray;
- (NSArray *)iptcKeywordArray;

//
// Time Range Support
//
+ (NSDictionary *)timeRangeInfoDictFromPhotoArray:(NSArray *)aPhotoArray;
- (NSCalendarDate *)nearestCalendarDate;
- (NSInteger)monthIndexRelativeToReferenceDate:(NSCalendarDate *)aReferenceDate;

- (NSCalendarDate *)radiusCalendarDate;
- (NSNumber *)timeRadiusValue;
- (CGFloat)timeRadius;

// 
// iPhoto sync support
+ (NSArray *)allPhotoLibIDs;
+ (NSString *)iPhotoDuplicateFileDictPath;
+ (NSMutableDictionary *)iPhotoDuplicateFileDict;
+ (void)recordDuplicateChecksum:(MMSChecksum *)aChecksum
                       usingKey:(NSString *)aKey;

+ (BOOL)isNSImageFileTypeAtPath:(NSString *)aPath;

//
// Caching Support
//
+ (void)cacheBitmapsForPhotosInArray:(NSArray *)aPhotoArray;

// 
// Export Support
//
- (NSString *)thumbnailExportName;
+ (NSArray *)xmlSerializationKeysForWebExport;

// Geocoding conveniences
- (BOOL)hasLatLongValues;
- (BOOL)hasOriginalGPSValues;
- (NSString *)latitude;
- (NSString *)longitude;


//
// Update Support
//
+ (NSArray *)revisionInfoDictKeys;
+ (NSArray *)revisionInfoDictArrayWithQualifier:(id)aQualifier;
- (void)validateAndAdustIPhotoLocation;
- (NSURL *)iPhotoURL;
- (void)readAndStoreImageSize;
- (void)readAndStoreImageSizeUsingURL:(NSURL *)anURL;
- (BOOL)storeThumbnailFromURL:(NSURL *)aThumbnailURL;

// 
// Folder sync support
+ (NSArray *)uniqueNewURLArrayAtFolderPath:(NSString *)aPath
                           recursiveSearch:(BOOL)aRecursiveChoice;

//
// Photo proxies
//
+ (NSArray *)proxiesForActivePhotoCreatedSinceDate:(NSCalendarDate *)aDate;
+ (NSArray *)proxiesForActivePhotosFromLastImport;
+ (NSArray *)proxiesForAllPhotos;
+ (NSArray *)proxiesForAllPhotosWithALocation;
+ (NSArray *)proxiesForAllPhotosWithPeople;
+ (NSArray *)proxiesForAllActivePhotos;
+ (NSArray *)proxiesWithQualifier:(id)aQualifier;
+ (NSArray *)proxiesWithSQLWhereClause:(NSString *)aSQLWhereClause;
+ (NSArray *)objectIDArrayForAllPhotos;

//
// Filtering support
//

+ (NSArray *)allPhotosIDsWithNoDate;
+ (NSArray *)allPhotosIDsWithADate;
+ (NSArray *)allPhotosIDsWithNoLocation;
+ (NSArray *)allPhotosIDsWithALocation;
+ (NSArray *)allPhotosIDsWithNoPeople;
+ (NSArray *)allPhotosIDsWithPeople;
+ (NSArray *)allPhotosIDsWithNoNotes;
+ (NSArray *)allPhotosIDsWithNotes;
+ (NSArray *)allPhotosIDsWithNoAttachments;
+ (NSArray *)allPhotosIDsWithAttachments;
+ (NSArray *)allPhotoIDsUsingLikeString:(NSString *)aLikeString;
+ (NSArray *)objectIDArrayWithSQLWhereClause:(NSString *)aWhereClause;

+ (NSArray *)activePhotosFromStartDate:(NSDate *)aStartDate
						throughEndDate:(NSDate *)anEndDate;

+ (NSArray *)photosInArray:(NSArray *)aPhotoArray
			 fromStartDate:(NSDate *)aStartDate
			throughEndDate:(NSDate *)anEndDate;

+ (NSArray *)photosWithPerson:(MMSPerson *)aPerson 
				   atLocation:(MMSLocation *)aLocation;

//
// Metadata mapping
//
- (NSDictionary *)rawIPTCDataDict;
- (NSDictionary *)xmpDataDictFromFile;
- (void)performMappingWithIPhotoFacesInfoDictArray:(NSArray *)aFacesInfoDictArray;
- (void)performMappingWithRawIPTCDataDict:(NSDictionary *)aDataDict;
- (void)storeDataFromGPSDataDict:(NSDictionary *)aDataDict;
- (void)storeDataFromIPhotoGPSDataDict:(NSDictionary *)aDataDict;
- (void)performMappingWithRawXMPDataDict:(NSDictionary *)aDataDict;
- (NSMutableDictionary *)personQualDictFromNameString:(NSString *)aString;
- (void)setLocationUsingInfoDict:(NSDictionary *)anInfoDict;

// 
// fotoFlag Support
//
- (NSDictionary *)fotoFlagDataDict;
- (NSImage *)fotoFlagImage;

// 
// Utility
//
+ (unsigned long long)totalFileSizeForPhotos:(NSArray *)aPhotoArray;
+ (unsigned long long)totalFileSizeForAllPhotos;
+ (unsigned long long)totalFileSizeForAllActivePhotos;

+ (NSArray *)photoIDArrayForPhotosWithAssignedPeople;
+ (NSArray *)activePhotosObjectIDArray;
+ (void)configureAssignedPersonCacheDict;
+ (BOOL)photoWithIDHasAssignedPeople:(NSString *)anID;

- (void)updateAssignedPersonStatus;

- (BOOL)isNSImageFileType;
- (BOOL)isQuickTimeMovieFileType;

// Thumbnail support
+ (NSURL *)derivedThumbnailURLForObjectID:(NSString *)anObjectID;
+ (NSString *)derivedThumbnailPathForObjectID:(NSString *)anObjectID;
+ (NSString *)derivedThumbnailPathForChecksumString:(NSString *)aChecksumString;
- (NSString *)thumbnailPath;
- (NSString *)checksumBasedThumbnailPath;

+ (NSURL *)missingThumbnailURL;
- (NSURL *)_photoThumbnailURL;

+ (void)createThumbnailForPhotoWithObjectID:(NSString *)anObjectID;

- (NSURL *)derivedThumbnailURL;
- (BOOL)createAndStoreThumbnail;
- (void)createAndStoreBitmapCache;
- (void)refreshArchivedIconImageRep;
- (NSURL *)missingThumbnailURL;
- (BOOL)hasUnmountedFileSystem;

/*
 MMSCoverflowImage protocol
 */

- (NSString *)coverflowImagePath;
- (NSString *)lowRezCoverflowImagePath;


//
// Fuzzy Date Support
//
+ (NSInteger)monthIntervalBetweenStartPhoto:(MMSPhoto *)aStartPhoto 
                          andEndPhoto:(MMSPhoto *)anEndPhoto;
+ (NSArray *)shortMonthArray;
- (NSInteger)monthAsInt;
- (NSInteger)dayAsInt;
- (NSInteger)yearAsInt;
- (NSString *)yearMonthString;


- (NSString *)monthYearDisplayString;
- (NSString *)monthDayYearDisplayString;
- (NSString *)localizedMonthDisplayString;
- (BOOL)hasPreciseYear;
- (BOOL)hasPreciseMonth;
- (BOOL)hasPreciseDay;
- (BOOL)hasMoreThanOneMarker;

- (NSString *)iso8601CreationDateString;

- (NSString *)locationDisplayString;

//
// Export
//
- (BOOL)exportJPEGToPath:(NSString *)aPath
            scaledToSize:(NSSize)aSize;

- (BOOL)exportJPEGToPath:(NSString *)aPath
            scaledToSize:(NSSize)aSize
		compressionLevel:(NSNumber *)aCompressionLevel;


- (BOOL)exportThumbnailToPath:(NSString *)aBasePath
                         size:(NSSize)aSize;

+ (void)writeXMLExportPackageForPhotos:(NSArray *)aPhotoArray
                                toPath:(NSString *)aPath
                     optionsDictionary:(NSDictionary *)anOptionsDict
                              delegate:(id)aDelegate;

+ (void)writeRSSExportPackageForPhotos:(NSArray *)aPhotoArray
                                toPath:(NSString *)aPath
                     optionsDictionary:(NSDictionary *)anOptionsDict
                              delegate:(id)aDelegate;


//
// Conveninences
//
- (NSString *)formattedSize;
- (NSInteger)personCount;
- (NSMutableDictionary *)latLongValueDict;

//
// Library management support
//
- (BOOL)copyFileToLibraryFolder;
- (NSURL *)photoURLUsingDBRecord;
- (BOOL)copyFileToLibraryFolder;
- (NSString *)storagePathInLibraryFolder;

- (NSArray *)childObjectsToDelete;

//
// Utilities
//
- (NSImageRep *)generatedIconImageRep;
- (void)storeIconImageRep:(NSImageRep *)anImageRep;


//
// Web Services Support
//
- (void)updateUsingRemoteMetaDataDict:(NSDictionary *)aDataDict;

// Accessors
- (BOOL)bitmapCacheIsLoaded;
- (void)setBitmapCacheIsLoaded:(BOOL)flag;

- (MMSChecksum *)checksum;
- (void)setChecksum:(MMSChecksum *)aChecksum;

- (NSString *)checksumString;
- (void)setChecksumString:(NSString *)aChecksumString;

- (NSString *)originalChecksumString;
- (void)setOriginalChecksumString:(NSString *)aString;

- (NSString *)fileURL;
- (void)setFileURL:(NSString *)aFileURL;

- (NSDate *)catalogDate;
- (void)setCatalogDate:(NSDate *)aCatalogDate;

- (NSCalendarDate *)creationDate;
- (void)setCreationDate:(NSCalendarDate *)aCreationDate;

- (NSString *)creationDateString;
- (void)setCreationDateString:(NSString *)aString;

- (NSNumber *)exportScaleFactor;
- (void)setExportScaleFactor:(NSNumber *)anExportScaleFactor;

- (NSArray *)faceInfoDictArray;
- (void)setFaceInfoDictArray:(NSArray *)anArray;

- (NSNumber *)fileSize;
- (void)setFileSize:(NSNumber *)aFileSize;

- (BOOL)hasMissingSourceFile;
- (void)setHasMissingSourceFile:(BOOL)flag;

- (BOOL)hasUnsyncedXMPData;
- (void)setHasUnsyncedXMPData:(BOOL)flag;

- (NSString *)isHidden;
- (void)setIsHidden:(NSString *)value;

- (NSDate *)lastUpdate;
- (void)setLastUpdate:(NSDate *)aLastUpdate;

- (NSString *)locationID;
- (void)setLocationID:(NSString *)aLocationID;

- (MMSLocation *)location;
- (void)setLocation:(MMSLocation *)aLocation;

- (NSString *)photoLibID;
- (void)setPhotoLibID:(NSString *)anID;

- (NSString *)lowRezURL;
- (void)setLowRezURL:(NSString *)aLowRezURL;

- (NSImage *)miniIconImage;
- (void)setMiniIconImage:(NSImage *)anImage;

- (NSString *)notes;
- (void)setNotes:(NSString *)value;

- (NSAttributedString *)notesAttributedString;
- (void)setNotesAttributedString:(NSAttributedString *)aNotesAttributedString;

- (NSNumber *)numberOfPeople;
- (void)setNumberOfPeople:(NSNumber *)aNumber;

- (NSString *)title;
- (void)setTitle:(NSString *)aTitle;

- (MMSPerson *)photoOwner;
- (void)setPhotoOwner:(MMSPerson *)aPhotoOwner;

- (NSString *)photoOwnerID;
- (void)setPhotoOwnerID:(NSString *)aPhotoOwnerID;

- (NSURL *)photoThumbnailURL;
- (void)setPhotoThumbnailURL:(NSURL *)aPhotoThumbnailURL;

- (NSNumber *)gpsAltitude;
- (void)setGpsAltitude:(NSNumber *)value;

- (NSNumber *)gpsLatitude;
- (void)setGpsLatitude:(NSNumber *)value;

- (NSNumber *)gpsLongitude;
- (void)setGpsLongitude:(NSNumber *)value;

- (NSNumber *)pixelsWide;
- (void)setPixelsWide:(NSNumber *)aPixelsWide;

- (NSNumber *)pixelsHigh;
- (void)setPixelsHigh:(NSNumber *)aPixelsHigh;

- (NSString *)thumbnailURL;
- (void)setThumbnailURL:(NSString *)aThumbnailURL;

- (NSImage *)iconImage;
- (void)setIconImage:(NSImage *)anIconImage;

- (NSImageRep *)iconImageRep;
- (void)setIconImageRep:(NSImageRep *)anIconImageRep;

- (NSImage *)previewImage;
- (void)setPreviewImage:(NSImage *)aPreviewImage;

- (NSDictionary *)rawXMPDataDict;
- (void)setRawXMPDataDict:(NSDictionary *)aDataDict;

- (NSArray *)selectionMarkers;
- (void)setSelectionMarkers:(NSArray *)anArray;

- (NSString *)yearString;
- (void)setYearString:(NSString *)aString;

- (NSString *)monthString;
- (void)setMonthString:(NSString *)aString;

- (NSString *)dayString;
- (void)setDayString:(NSString *)aString;

- (NSString *)relativeStoragePath;
- (void)setRelativeStoragePath:(NSString *)aPath;

//
// Caching support
//

- (NSString *)archivedIconImageRepPath;
- (NSImageRep *)archivedIconImageRep;
- (void)storeIconImageRep;
- (void)unloadBitmapCache;
+ (void)clearAllCacheDictionaries;

//
// iPhoto support
//
+ (iPhotoLibraryController *)iPhotoLibraryController;
+ (void)synchronizeWithIPhotoLibrary;
+ (BOOL)iPhotoLibraryHasNewEntries;
+ (NSArray *)uniqueNewIPhotoKeys;
+ (void)synchronizeWithIPhotoLibraryForOwner:(MMSPerson *)aPhotoOwner;
+ (MMSPhoto *)newOrExistingPhotoFromIPhotoKey:(NSString *)anIPhotoKey
                                    forOwner:(MMSPerson *)aPhotoOwner;
+ (NSSize)imageSizeForFileAtURL:(NSURL *)anURL;

//
// Flickr
//

- (NSArray *)flickrDateArgsForPhotoID:(NSString *)aPhotoID;

// Conveniences
- (BOOL)assignPersonToPhoto:(MMSPerson *)aPerson;
- (BOOL)fileIsReadable;
- (BOOL)fileIsWriteable;
- (NSString *)fileName;
- (void)deactivate;
- (void)reactivate;
- (NSArray *)specificSelectionMarkers;
- (NSArray *)selectionMarkers;
- (NSArray *)selectionMarkerThumbnaiImageArray;
- (MMSSelectionMarker *)markerForPerson:(MMSPerson *)aPerson;
- (NSArray *)personArray;
- (NSImage *)image;
- (NSSize)imageSize;
- (NSImage *)thumbnailImage;
- (NSImage *)doubleSizedIconImage;
- (NSImage *)doubleSizedThumbnailImage;
- (NSURL *)photoURL;
- (NSURL *)photoThumbnailURL;
- (BOOL)hasClonableMetaData;
- (BOOL)hasPeople;
- (void)cloneMetaDataFromPhoto:(MMSPhoto *)aPhoto;
- (void)writePreviewImageToURL:(NSURL *)anURL;
- (NSImage *)imageScaledToRect:(NSRect)aRect;
- (NSRect)rectScaledToFitInRect:(NSRect)aRect;
- (CGFloat)scaleFactorForSize:(NSSize)aSize;
- (CGFloat)reductionToFitInRect:(NSRect)aRect;
- (NSImageRep *)iconImageRep;
- (NSImageRep *)previewImageRep;
- (NSMutableArray *)mutableDateComponentArray;
- (NSArray *)photoCollectionLinks;

//
// Misc
//
- (NSSize)glTextureSize;

- (BOOL)hasRegionRect;

//
// Export
//
- (BOOL)copyFileToPath:(NSString *)aBasePath;
- (BOOL)copyFileToPath:(NSString *)aBasePath
          withFileName:(NSString *)aFileName;
- (BOOL)copyThumbnailToPath:(NSString *)aBasePath;
- (BOOL)copyMapFilesToPath:(NSString *)aBasePath;
- (void)copySelectionMarkerThumbnailsToPath:(NSString *)aBasePath;
- (void)copyMarkerPersonThumbnailsToPath:(NSString *)aBasePath;
- (void)copyMarkerPersonThumbnailsToPath:(NSString *)aBasePath
						   referenceDate:(NSDate *)aReferenceDate
					  existingThumbnails:(NSArray *)aThumbnailArray;

+ (NSString *)photosFolderName;
+ (NSString *)thumbnailsFolderName;
- (NSString *)relativeFilePath;
- (NSString *)storedOrDerivedRelativePath;
- (NSString *)relativeJPEGConvertedFilePath;
- (NSString *)relativeThumbnailPath;

// 
// MemoryMiner Web Service Support
//

- (NSMutableDictionary *)wsDataDictWithThumbnailURLBase:(NSString *)thumbnailURLBase
                                           photoURLBase:(NSString *)photoURLBase;

@end

#define photoLibraryLoadedNotification @"MMSPhotoLibraryLoadedNotification"
#define photoLibraryImportNeedeNotification @"MMSPhotoLibraryImportNeededNotification"
#define photoLibraryNeedsLoadingNotification @"MMSPhotoLibraryNeedsLoadingNotification"
#define finishEditNotification @"MMSPhotoFinishEditNotification"
#define newPhotosAddedNotification @"MMSPhotoNewPhotosAddedNotification"
