@import <AppKit/CPView.j>
@import "MKMapScene.j"
@import "MKMarker.j"
@import "MKLocation.j"
@import "MKPolyline.j"

/* a "class" variable that will hold the domWin.google.maps object/"namespace" */
var gmNamespace = nil;

@implementation CPWebView(ScrollFixes) {
    - (void)loadHTMLStringWithoutMessingUpScrollbars:(CPString)aString
    {
        [self _startedLoading];
    
        _ignoreLoadStart = YES;
        _ignoreLoadEnd = NO;
    
        _url = null;
        _html = aString;
    
        [self _load];
    }
}
@end

@implementation MKMapView : CPWebView
{
    CPString        _apiKey;
    DOMElement      _DOMMapElement;
    JSObject        _gMap               @accessors(property=gMap);
    MKMapScene      _scene              @accessors(property=scene);
	CPArray			markerArray			@accessors;
	CPArray			polylineArray		@accessors;
    BOOL            _mapReady			@accessors(property=mapReady);
    BOOL            _googleAjaxLoaded;
	MarkerManager   markerManager @accessors;
    JSObject        mapTypes @accessors;
    
    id delegate @accessors;
}

- (id)initWithFrame:(CGRect)aFrame apiKey:(CPString)apiKey
{
    _apiKey = apiKey;
    if (self = [super initWithFrame:aFrame]) {
        _scene = [[MKMapScene alloc] initWithMapView:self];

        var bounds = [self bounds];
        
		//[self setScrollMode:CPWebViewScrollAppKit];
		[self setMarkerArray:[CPArray array]];
		[self setPolylineArray:[CPArray array]];
		
		[self setScrollMode:CPWebViewScrollAppKit];
		
        [_scrollView setHasHorizontalScroller:NO];
        [_scrollView setHasVerticalScroller:NO];

        [self setFrameLoadDelegate:self];
        [self loadHTMLStringWithoutMessingUpScrollbars:@"<html><head><script type=\"text/javascript\" src=\"http://www.google.com/jsapi?key=" + _apiKey + "\"></script></head><body style='padding:0px; margin:0px'><div id='MKMapViewDiv' style='left: 0px; top: 0px; width: 100%; height: 100%'></div></body></html>"];
    }

    return self;
}


- (void)webView:(CPWebView)aWebView didFinishLoadForFrame:(id)aFrame 
{
    var head = document.getElementsByTagName("head").item(0);
    var scriptTag = document.createElement("script");
    
    scriptTag.setAttribute("type", "text/javascript");
    scriptTag.setAttribute("charset", "utf-8");
    scriptTag.setAttribute("src", "http://gmaps-utility-library.googlecode.com/svn/trunk/markermanager/release/src/markermanager.js");
        
    //head.appendChild(scriptTag);

    [self loadGoogleMapsWhenReady];
}

- (void)loadGoogleMapsWhenReady() {
    var domWin = [self DOMWindow];
    
    if (typeof(domWin.google) === 'undefined') {
        domWin.window.setTimeout(function() {[self loadGoogleMapsWhenReady];}, 100);
    } else {
        var mapsJsLoaded = function () {
            //alert('mapsJsLoaded!');
            _googleAjaxLoaded = YES;
            _DOMMapElement = domWin.document.getElementById('MKMapViewDiv');
            [self createMap];
        };

		domWin.google.load('maps', '2', {'callback': mapsJsLoaded});
    }
}

- (void)createMap
{
    var domWin = [self DOMWindow];
    //remember the google maps namespace, but only once because it's a class variable
    if (!gmNamespace) {
        gmNamespace = domWin.google.maps;
    }
    
    // for some things the current google namespace needs to be used...
    var localGmNamespace = domWin.google.maps;

    //console.log("Creating map");
    _gMap = new localGmNamespace.Map2(_DOMMapElement);
    _gMap.addMapType(domWin.G_PHYSICAL_MAP);
    //_gMap.addMapType(gmNamespace.G_PHYSICAL_MAP)
    //_gMap.setMapType(localGmNamespace.G_PHYSICAL_MAP);
    //_gMap.setUIToDefault();
    _gMap.enableContinuousZoom();
    _gMap.enableScrollWheelZoom();
    
    // Google contstants aren't in the name space, but are
    // added to the window. Who knows why.
    mapTypes = domWin.G_DEFAULT_MAP_TYPES;
    mapTypes.push(domWin.G_PHYSICAL_MAP);
        
    localGmNamespace.Event.addListener(_gMap, 'zoomend', function(oldLevel, newLevel) 
        {
            [self informDelegateOfZoom];
        }
    );
        
    _gMap.setCenter(new localGmNamespace.LatLng(52, -1), 8);
    _gMap.setZoom(6);
    
    //markerManager = new MarkerManager(_gMap);
    
    //log('markerManager is ' +markerManager);
     
    //zoomend(oldLevel:Number, newLevel:Number)
    // Hack to get mouse up event to work
    localGmNamespace.Event.addDomListener(document.body, 'mouseup', function() { try { localGmNamespace.Event.trigger(domWin, 'mouseup'); } catch(e){} });

    _mapReady = YES;

    if (delegate && [delegate respondsToSelector:@selector(mapViewIsReady:)]) {
        [delegate mapViewIsReady:self];
    }
}

- (void)informDelegateOfZoom
{
    if (delegate && ([delegate respondsToSelector:@selector(mapDidChangeZoomLevel:)]))
    {
        [delegate performSelector:@selector(mapDidChangeZoomLevel:) 
                       withObject:_gMap.getZoom()];
    }
}

- (void)setFrameSize:(CGSize)aSize
{
    [super setFrameSize:aSize];
    var bounds = [self bounds];
    if (_gMap) {
        _gMap.checkResize();
    }
}

- (void)setFrame:(CGRect)aFrame
{	
    [super setFrame:aFrame];
	
	[_frameView setFrameSize:aFrame.size];
	
	var bounds = [self bounds];

    if (_gMap) {
        _gMap.checkResize();
	}
}

/* Overriding CPWebView's implementation */
- (BOOL)_resizeWebFrame {
    var width = [self bounds].size.width,
        height = [self bounds].size.width;

    _iframe.setAttribute("width", width);
    _iframe.setAttribute("height", height);

    [_frameView setFrameSize:CGSizeMake(width, height)];
}

- (void)viewDidMoveToSuperview
{
    if (!_mapReady && _googleAjaxLoaded) {
        [self createMap];
    }
    [super viewDidMoveToSuperview];
}

- (void)setCenter:(MKLocation)aLocation {
    if (_mapReady) {
        _gMap.setCenter([aLocation googleLatLng]);
    }
}

- (void)setZoom:(int)zoomLevel {
    if (_mapReady) {
        _gMap.setZoom(zoomLevel);
    }
}

- (MKMarker)addMarker:(MKMarker)aMarker atLocation:(MKLocation)aLocation
{
    if (_mapReady) {
        var gMarker = [aMarker gMarker];
        gMarker.setLatLng([aLocation googleLatLng]);
        _gMap.addOverlay(gMarker);
		
		[markerArray addObject:aMarker];
		
    } else {
        // TODO some sort of queue?
    }
    return marker;
}

- (void)clearOverlays 
{
    if (_mapReady) {
        _gMap.clearOverlays();
    }
}

- (void)hideAllMarkers
{	
	[self showOrHideAllMarkers:YES];
}

- (void)showAllMarkers
{
	[self showOrHideAllMarkers:NO];
}

- (void)showOrHideAllMarkers:(BOOL)shouldHide
{
    if (_mapReady) 
	{		
		var myEnum = [markerArray objectEnumerator];
		var myMarker = nil;
		
		while (myMarker = [myEnum nextObject])
		{				
			[myMarker setHidden:shouldHide];
		}
	}
}

- (void)hideAllPolylines
{		
	[self showOrHideAllPolylines:YES];
}

- (void)showAllPolylines
{
	[self showOrHideAllPolylines:NO];
}

- (void)showOrHideAllPolylines:(BOOL)shouldHide
{
    if (_mapReady) 
	{		
		var myEnum = [polylineArray objectEnumerator];
		var myPolyline = nil;
		
		while (myPolyline = [myEnum nextObject])
		{				
			[myPolyline setHidden:shouldHide];
		}
	}
}

- (void)addMapItem:(MKMapItem)mapItem
{
    [mapItem addToMapView:self];
}

- (BOOL)isMapReady 
{
    return _mapReady;
}

+ (JSObject)gmNamespace {
    return gmNamespace;
}

@end

